@@ -26,8 +26,10 @@ USER = environ['SUDO_USER'] if 'SUDO_USER' in environ else environ['USER']
 PATH = environ['PATH']
 KEYRINGDIR = path.join(path.expanduser('~' + USER), '.meile-gui')
 BASEDIR  = path.join(path.expanduser('~' + USER), '.sentinelcli')
-
 APIURL   = "https://api.sentinel.mathnodes.com"
+SERVER_URL = "https://aimokoivunen.mathnodes.com:5000"
+NODE_SCORE_ENDPOINT = "/api/nodescores"
+NODE_LOCATION_ENDPOINT = "/api/nodelocations"
 
 NodesInfoKeys = ["Moniker","Address","Provider","Price","Country","Speed","Latency","Peers","Handshake","Version","Status"]
 SubsInfoKeys = ["ID", "Owner", "Plan", "Expiry", "Denom", "Node", "Price", "Deposit", "Free", "Status"]
@@ -37,9 +39,12 @@ dash = "-"
 
 MeileConfig = MeileGuiConfig()
 sentinelcli = MeileConfig.resource_path("../bin/sentinelcli")
+RPC = "https://rpc.mathnodes.com:443"
 
 class NodeTreeData():
     NodeTree = None
+    NodeScores = {}
+    NodeLocations = {}
     
     def __init__(self, node_tree):
         if not node_tree:
@@ -51,7 +56,7 @@ class NodeTreeData():
     def get_nodes(self, latency, *kwargs):
         AllNodesInfo = []
         print("Running sentinel-cli with latency: %s" % latency)
-        nodeCMD = [sentinelcli, "query", "nodes", "--node", "https://rpc.mathnodes.com:443", "--limit", "20000", "--timeout", "%s" % latency]
+        nodeCMD = [sentinelcli, "query", "nodes", "--node", RPC, "--limit", "20000", "--timeout", "%s" % latency]
     
         proc = Popen(nodeCMD, stdout=PIPE)
         
@@ -112,6 +117,45 @@ class NodeTreeData():
                 pass
             
         self.NodeTree.show()
+        self.GetNodeScores()
+        self.GetNodeLocations()
+        
+    def GetNodeScores(self):
+        try:
+            r = requests.get(SERVER_URL + NODE_SCORE_ENDPOINT)
+            data = r.json()
+          
+            for nlist in data['data']:
+                print(nlist)
+                k=0
+                for nd in nlist:
+                   if k == 0:
+                       self.NodeScores[nlist[k]] = [nlist[k+1], nlist[k+2]]
+                       k += 1
+                   else:
+                       break
+            print(self.NodeScores)
+        except Exception as e:
+            print(str(e)) 
+            
+    def GetNodeLocations(self):
+        try:
+            r = requests.get(SERVER_URL + NODE_LOCATION_ENDPOINT)
+            data = r.json()
+          
+            for nlist in data['data']:
+                k=0
+                for nd in nlist:
+                   if k == 0:
+                       self.NodeLocations[nlist[k]] = nlist[k+1]
+                       k += 1
+                   else:
+                       break
+            #print(self.NodeLocations)
+        except Exception as e:
+            print(str(e)) 
+            
+             
     
     def CreateNodeTreeStructure(self, **kwargs):
         NodeTreeBase = Tree()
@@ -137,7 +181,7 @@ class NodeTreeData():
         SubsNodesInfo = []
         SubsFinalResult    = []
         print("Geting Subscriptions... %s" % ADDRESS)
-        subsCMD = [sentinelcli, "query", "subscriptions", "--node", "https://rpc.mathnodes.com:443", "--status", "Active", "--limit", "100", "--address" ,ADDRESS]
+        subsCMD = [sentinelcli, "query", "subscriptions", "--node", RPC, "--status", "Active", "--limit", "100", "--address" ,ADDRESS]
         proc = Popen(subsCMD, stdout=PIPE)
     
         k=1
@@ -176,29 +220,11 @@ class NodeTreeData():
                                             })
                 print("Sub not found in list")
                 k += 1
-                continue   
-            quotaCMD = [sentinelcli, 'query', 'quotas', '--node', 'https://rpc.mathnodes.com:443', '--page', '1', SubsResult[SubsInfoKeys[0]][k]]
-            proc = Popen(quotaCMD, stdout=PIPE)
-                    
-            h=1
-            for line in proc.stdout.readlines():
-                #print(line)
-                if h < 4:
-                    h += 1 
-                    continue
-                if h >=4 and '+-----------+' in str(line.decode('utf-8')):
-                    break
-                else:
-                    nodeQuota = str(line.decode('utf-8')).split("|")[2:-1]
-                    allotted = float(re.findall(r'[0-9]+\.[0-9]+', nodeQuota[0])[0])
-                    consumed = float(re.findall(r'[0-9]+\.[0-9]+', nodeQuota[1])[0])
-                    
-                    if allotted == consumed:
-                        break
-                    else:
-                        
-                            
-                        SubsFinalResult.append({
+                continue
+            
+            nodeQuota = self.GetQuota(SubsResult[SubsInfoKeys[0]][k])
+            if nodeQuota:
+                SubsFinalResult.append({
                                             FinalSubsKeys[0] : SubsResult[SubsInfoKeys[0]][k],
                                             FinalSubsKeys[1] : NodeData[NodesInfoKeys[0]],
                                             FinalSubsKeys[2] : SubsResult[SubsInfoKeys[5]][k],
@@ -208,22 +234,43 @@ class NodeTreeData():
                                             FinalSubsKeys[6] : nodeQuota[0],
                                             FinalSubsKeys[7] : nodeQuota[1]
                                             })
-           
             k += 1 
 
-        return SubsFinalResult
-
+        return SubsFinalResult   
 
 
+    def GetQuota(self, id):
+        quotaCMD = [sentinelcli, 'query', 'quotas', '--node', RPC, '--page', '1', id]
+        proc = Popen(quotaCMD, stdout=PIPE)
+        h=1
+        for line in proc.stdout.readlines():
+            #print(line)
+            if h < 4:
+                h += 1 
+                continue
+            if h >=4 and '+-----------+' in str(line.decode('utf-8')):
+                break
+            else:
+                nodeQuota = str(line.decode('utf-8')).split("|")[2:-1]
+                allotted = float(re.findall(r'[0-9]+\.[0-9]+', nodeQuota[0])[0])
+                consumed = float(re.findall(r'[0-9]+\.[0-9]+', nodeQuota[1])[0])
+                
+                if allotted == consumed:
+                    return None
+                else:
+                    return nodeQuota
+                
+        
 def get_node_infos(naddress):
     endpoint = "/nodes/" + naddress
     
     NodeInfoDict = {}
-    
-    r = requests.get(APIURL + endpoint)
-    
-    remote_url = r.json()['result']['node']['remote_url']
-    
+    try: 
+        r = requests.get(APIURL + endpoint)
+        
+        remote_url = r.json()['result']['node']['remote_url']
+    except Exception as e:
+        print(str(e))
     
 
 def disconnect():
@@ -240,7 +287,7 @@ def disconnect():
     #wg_downCMD = ['wg-quick', 'down', CONFFILE]
         
     proc1 = Popen(partCMD)
-    proc1.wait(timeout=10)
+    proc1.wait(timeout=30)
     
     #proc = Popen(wg_downCMD, stdout=PIPE, stderr=PIPE)
     #proc_out,proc_err = proc.communicate()
