@@ -1,12 +1,12 @@
 from kivy.properties import BooleanProperty, StringProperty
 from kivy.uix.recycleview.views import RecycleDataViewBehavior
-from kivy.uix.label import Label
 from kivymd.uix.label import MDLabel
 from kivymd.uix.card import MDCard
 from kivymd.uix.dialog import MDDialog
-from kivymd.uix.button import MDFlatButton, MDRaisedButton,MDFillRoundFlatButton
+from kivymd.uix.button import MDFlatButton, MDRaisedButton, MDFillRoundFlatButton
 from kivy.uix.recycleview import RecycleView
 from kivy.uix.boxlayout import BoxLayout
+from kivymd.uix.boxlayout import MDBoxLayout
 from kivymd.uix.menu import MDDropdownMenu
 from kivymd.uix.list import OneLineIconListItem
 from kivy.metrics import dp
@@ -19,11 +19,14 @@ from kivy.core.window import Window
 from kivymd.uix.behaviors.elevation import RectangularElevationBehavior
 from kivy.core.clipboard import Clipboard
 from kivy.animation import Animation
+from kivy.clock import Clock
+
 
 from functools import partial
 from urllib3.exceptions import InsecureRequestWarning
 import requests
 import re
+import psutil
 from os import path
 from subprocess import Popen, TimeoutExpired
 
@@ -32,8 +35,14 @@ from cli.sentinel import IBCCOINS
 from typedef.win import CoinsList, WindowNames
 from conf.meile_config import MeileGuiConfig
 from cli.wallet import HandleWalletFunctions
+from cli.sentinel import NodeTreeData
 import main.main as Meile
 
+SERVER_ADDRESS      = "https://aimokoivunen.mathnodes.com:5000"
+APIURL              = "https://api.sentinel.mathnodes.com"
+API_PING_ENDPOINT   = "/api/ping"
+API_RATING_ENDPOINT = "/api/rating"
+
 class WalletInfoContent(BoxLayout):
     def __init__(self, seed_phrase, name, address, password, **kwargs):
         super(WalletInfoContent, self).__init__(**kwargs)
@@ -59,7 +68,35 @@ class WalletInfoContent(BoxLayout):
         anim.start(label)
             
         return label
-
+    
+class RatingContent(MDBoxLayout):
+    naddress = StringProperty()
+    moniker  = StringProperty()
+    
+    def __init__(self, moniker, naddress):
+        super(RatingContent, self).__init__()
+        self.naddress = naddress
+        self.moniker  = moniker
+    
+    
+    def SubmitRating(self, rating, node_address):
+        UUID = Meile.app.root.get_screen(WindowNames.PRELOAD).UUID
+        try:
+            rating_dict = {'uuid' : "%s" % UUID, 'address' : "%s" % node_address, "rating" : rating}
+            ping = requests.post(SERVER_ADDRESS + API_RATING_ENDPOINT, json=rating_dict)
+            if ping.status_code == 200:
+                print("Rating Sent")
+            else:
+                print("Error submitting rating")
+        except Exception as e:
+            print(str(e))
+            pass
+        
+    def return_rating_value(self):
+        return self.ids.rating_slider.value
+    
+    
+    
 class SubscribeContent(BoxLayout):
     
     
@@ -132,34 +169,6 @@ class SubscribeContent(BoxLayout):
 class IconListItem(OneLineIconListItem):
     icon = StringProperty()
 
-   
-class SelectableLabel(RecycleDataViewBehavior, Label):
-    ''' Add selection support to the Label '''
-    index = None
-    selected = BooleanProperty(False)
-    selectable = BooleanProperty(True)
-
-    def refresh_view_attrs(self, rv, index, data):
-        ''' Catch and handle the view changes '''
-        self.index = index
-        return super(SelectableLabel, self).refresh_view_attrs(
-            rv, index, data)
-
-    def on_touch_down(self, touch):
-        ''' Add selection on touch down '''
-        if super(SelectableLabel, self).on_touch_down(touch):
-            return True
-        if self.collide_point(*touch.pos) and self.selectable:
-            return self.parent.select_with_touch(self.index, touch)
-
-    def apply_selection(self, rv, index, is_selected):
-        ''' Respond to the selection of items in the view. '''
-        self.selected = is_selected
-        if is_selected:
-            print("selection changed to {0}".format(rv.data[index]))
-        else:
-            print("selection removed for {0}".format(rv.data[index]))
-
 
 class NodeRV(RecycleView):    
     pass
@@ -180,6 +189,10 @@ class RecycleViewRow(MDCard,RectangularElevationBehavior,ThemableBehavior, Hover
     text = StringProperty()    
     dialog = None
     
+    def get_font(self):
+        Config = MeileGuiConfig()
+        return Config.resource_path("../fonts/arial-unicode-ms.ttf")
+    
     def on_enter(self, *args):
         self.md_bg_color = get_color_from_hex("#200c3a")
         Window.set_system_cursor('hand')
@@ -189,25 +202,28 @@ class RecycleViewRow(MDCard,RectangularElevationBehavior,ThemableBehavior, Hover
         Window.set_system_cursor('arrow')
         
     def get_city_of_node(self, naddress):   
-        APIURL   = "https://api.sentinel.mathnodes.com"
+        
 
         endpoint = "/nodes/" + naddress.lstrip().rstrip()
-        print(APIURL + endpoint)
-        r = requests.get(APIURL + endpoint)
-        remote_url = r.json()['result']['node']['remote_url']
-        r = requests.get(remote_url + "/status", verify=False)
-        print(remote_url)
-        requests.packages.urllib3.disable_warnings(category=InsecureRequestWarning)
-
-        NodeInfoJSON = r.json()
-        NodeInfoDict = {}
-        
-        NodeInfoDict['connected_peers'] = NodeInfoJSON['result']['peers']
-        NodeInfoDict['max_peers']       = NodeInfoJSON['result']['qos']['max_peers']
-        NodeInfoDict['version']         = NodeInfoJSON['result']['version']
-        NodeInfoDict['city']            = NodeInfoJSON['result']['location']['city']
-
+        #print(APIURL + endpoint)
+        try:
+            requests.packages.urllib3.disable_warnings(category=InsecureRequestWarning)
+            r = requests.get(APIURL + endpoint)
+            remote_url = r.json()['result']['node']['remote_url']
+            r = requests.get(remote_url + "/status", verify=False)
+            print(remote_url)
+    
+            NodeInfoJSON = r.json()
+            NodeInfoDict = {}
+            
+            NodeInfoDict['connected_peers'] = NodeInfoJSON['result']['peers']
+            NodeInfoDict['max_peers']       = NodeInfoJSON['result']['qos']['max_peers']
+            NodeInfoDict['version']         = NodeInfoJSON['result']['version']
+            NodeInfoDict['city']            = NodeInfoJSON['result']['location']['city']
 
+        except Exception as e:
+            print(str(e))
+            return None
 
 
         if not self.dialog:
@@ -263,7 +279,7 @@ Node Version: %s
         self.dialog = None
         self.dialog = MDDialog(title="Subscribing...\n\n%s\n %s" %( deposit, sub_node[1]),md_bg_color=get_color_from_hex("#0d021b"))
         self.dialog.open()
-        yield 2.0
+        yield 0.6
 
         CONFIG = MeileGuiConfig.read_configuration(MeileGuiConfig, MeileGuiConfig.CONFFILE)        
         KEYNAME = CONFIG['wallet'].get('keyname', '')
@@ -335,6 +351,7 @@ Node Version: %s
         Meile.app.root.transition = SlideTransition(direction = "down")
         Meile.app.root.current = WindowNames.MAIN_WINDOW
         Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).SubResult = None
+        #Change this to switch_tab by ids
         Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).on_tab_switch(None,None,None,"Subscriptions")
     
     def closeDialog(self, inst):
@@ -343,41 +360,70 @@ Node Version: %s
             self.dialog = None
         except Exception as e:
             print(str(e))
-            return
+            self.dialog = None
      
 class RecycleViewSubRow(MDCard,RectangularElevationBehavior):
     text = StringProperty()
     dialog = None
         
     def get_data_used(self, allocated, consumed, node_address):
-        try:         
+        try:
+            ''' Since this function is called when opening the Subscription tab,
+                we need to do a little house keeping for the card switches and the
+                data consumed
+            '''         
             if Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['node'] == node_address:
                 self.ids.node_switch.active = True
             else:
                 self.ids.node_switch.active = False
             
+            if not Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).clock and Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['id']:
+                print("Not clock()")
+                self.setQuotaClock(Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['id'],
+                                   Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['node'])
+            
+            #End house keeping
+            
             allocated = float(allocated.replace('GB',''))
+            consumed = self.compute_consumed_data(consumed)
             
-            if "GB" in consumed:
-                consumed  = float(consumed.replace('GB', ''))
-            elif "MB" in consumed:
-                consumed = float(float(consumed.replace('MB', '')) / 1024)
-            elif "KB" in consumed:
-                consumed = float(float(consumed.replace('KB', '')) / (1024*1024))
-            elif "0.00B" in consumed:
-                consumed = 0.0
-            else:
-                consumed = float(float(re.findall(r'[0-9]+\.[0-9]+', consumed)[0].replace('B', '')) / (1024*1024*1024))
             if allocated == 0:
                 self.ids.consumed_data.text = "0%"
                 return 0
             
             self.ids.consumed_data.text = str(round(float(float(consumed/allocated)*100),2)) + "%"
-            return float(float(consumed/allocated)*100)
+            return round(float(float(consumed/allocated)*100),3)
         except Exception as e:
             print(str(e))
             return float(50)
         
+    def compute_consumed_data(self, consumed):
+        if "GB" in consumed:
+            consumed  = float(consumed.replace('GB', ''))
+        elif "MB" in consumed:
+            consumed = float(float(consumed.replace('MB', '')) / 1024)
+        elif "KB" in consumed:
+            consumed = float(float(consumed.replace('KB', '')) / (1024*1024))
+        elif "0.00B" in consumed:
+            consumed = 0.0
+        else:
+            consumed = float(float(re.findall(r'[0-9]+\.[0-9]+', consumed)[0].replace('B', '')) / (1024*1024*1024))
+        
+        return consumed
+        
+    def connected_quota(self, allocated, consumed):        
+        if Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).CONNECTED:
+            allocated = float(allocated.replace('GB',''))
+            consumed  = self.compute_consumed_data(consumed)
+            Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).ids.quota_pct.text = str(round(float(float(consumed/allocated)*100),2)) + "%"
+            return round(float(float(consumed/allocated)*100),3)
+        else:
+            Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).ids.quota_pct.text = "0.00%"
+            Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).ids.quota.value    = 0
+            return float(0)
+            
+    
+    
     def add_loading_popup(self, title_text):
         self.dialog = None
         self.dialog = MDDialog(title=title_text,md_bg_color=get_color_from_hex("#0d021b"))
@@ -392,7 +438,7 @@ class RecycleViewSubRow(MDCard,RectangularElevationBehavior):
             return
         
     @delayable
-    def connect_to_node(self, ID, naddress, moniker, switchValue, *kwargs):
+    def connect_to_node(self, ID, naddress, moniker, switchValue, **kwargs):
         
         '''
            These two conditionals are needed to check
@@ -402,7 +448,13 @@ class RecycleViewSubRow(MDCard,RectangularElevationBehavior):
         '''
         if Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['switch'] and naddress == Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['node'] and not switchValue:
             print("DISCONNECTING!!!")
-            Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).disconnect_from_node()
+            try:
+                Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).clock.cancel()
+                Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).clock = None
+            except Exception as e:
+                print(str(e))
+            if Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).disconnect_from_node():
+                self.connected_quota(None, None)
             return True
         
         if Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).CONNECTED:
@@ -414,10 +466,8 @@ class RecycleViewSubRow(MDCard,RectangularElevationBehavior):
             yield 0.6
             UUID = Meile.app.root.get_screen(WindowNames.PRELOAD).UUID
             try:
-                SERVER_ADDRESS = "https://aimokoivunen.mathnodes.com:5000"
-                API_ENDPOINT   = "/api/ping"
                 uuid_dict = {'uuid' : "%s" % UUID, 'os' : "L"}
-                ping = requests.post(SERVER_ADDRESS + API_ENDPOINT, json=uuid_dict)
+                ping = requests.post(SERVER_ADDRESS + API_PING_ENDPOINT, json=uuid_dict)
                 if ping.status_code == 200:
                     print('ping')
                 else:
@@ -429,11 +479,25 @@ class RecycleViewSubRow(MDCard,RectangularElevationBehavior):
             connected = HandleWalletFunctions.connect(HandleWalletFunctions, ID, naddress)
             
             if connected:
-                Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['node'] = naddress
-                Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['switch'] = True
-                print("%s, %s" % (Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['node'],
-                                  Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['switch']
-                                 ))
+                from copy import deepcopy
+                
+                Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).CONNECTED               = True
+                Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['moniker']   = moniker
+                Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['node']      = naddress
+                Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['switch']    = True
+                Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['id']        = ID
+                Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['allocated'] = self.allocated_text
+                Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['consumed']  = self.consumed_text
+                Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['og_consumed']  = deepcopy(Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['consumed']) 
+                
+                if not ID in Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).PersistentBandwidth:
+                    Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).PersistentBandwidth[ID] = Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch
+                else:
+                    Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).PersistentBandwidth[ID]['og_consumed'] = deepcopy(Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).PersistentBandwidth[ID]['consumed'])
+                
+                
+                self.setQuotaClock(ID, naddress)
+
                 self.remove_loading_widget()
                 self.dialog = MDDialog(
                     title="Connected!",
@@ -460,6 +524,46 @@ class RecycleViewSubRow(MDCard,RectangularElevationBehavior):
                                 on_release=partial(self.call_ip_get, False, "")
                             ),])
                 self.dialog.open()
+    
+    def setQuotaClock(self,ID, naddress):
+        self.UpdateQuotaForNode(Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['id'],
+                                Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['node'],
+                                None)
+        Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).clock = Clock.create_trigger(partial(self.UpdateQuotaForNode,
+                                                  Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['id'],
+                                                  Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['node']),
+                                                  120)
+        Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).clock()
+        
+    def UpdateQuotaForNode(self, ID, naddress, dt):
+        try:
+            print("%s: Getting Quota: " % ID, end= ' ')
+            startConsumption = Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).PersistentBandwidth[ID]['og_consumed']
+            Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).PersistentBandwidth[ID]['consumed'] = self.GetConsumedWhileConnected(self.compute_consumed_data(startConsumption))
+            
+            Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).ids.quota.value = self.connected_quota(Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).NodeSwitch['allocated'],
+                                                                                                      Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).PersistentBandwidth[ID]['consumed'])
+            print("%s,%s - %s%%" % (Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).PersistentBandwidth[ID]['consumed'],
+                                  startConsumption,
+                                  Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).ids.quota.value))
+        except Exception as e:
+            print("Error getting bandwidth!")
+            print(str(e))
+            
+        try: 
+            Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).clock()
+        except Exception as e:
+            print("Error running clock()")
+            print(str(e))
+            pass 
+        
+    def GetConsumedWhileConnected(self, sConsumed):
+        bytes_sent = round(float(float(psutil.net_io_counters(pernic=True)['wg99'].bytes_sent) / 1073741824),3)
+        bytes_recvd = round(float(float(psutil.net_io_counters(pernic=True)['wg99'].bytes_recv) / 1073741824),3)
+        
+        total_data = str(round(float(bytes_sent + bytes_recvd)+ float(sConsumed),3)) + "GB"
+        
+        return total_data    
                 
     def call_ip_get(self,result, moniker,  *kwargs):
         if result:
